@@ -34,7 +34,7 @@
 
 #include "pikepdf.h"
 
-std::string objecthandle_scalar_value(QPDFObjectHandle h, bool escaped)
+std::string objecthandle_scalar_value(QPDFObjectHandle h)
 {
     std::ostringstream ss;
     ss.imbue(std::locale::classic());
@@ -75,54 +75,44 @@ std::string objecthandle_pythonic_typename(QPDFObjectHandle h)
 
     switch (h.getTypeCode()) {
     case QPDFObject::object_type_e::ot_name:
-        ss << "pikepdf."
-           << "Name";
+        ss << "pikepdf.Name";
         break;
     case QPDFObject::object_type_e::ot_string:
-        ss << "pikepdf."
-           << "String";
+        ss << "pikepdf.String";
         break;
     case QPDFObject::object_type_e::ot_operator:
-        ss << "pikepdf."
-           << "Operator";
+        ss << "pikepdf.Operator";
         break;
     // LCOV_EXCL_START
     case QPDFObject::object_type_e::ot_inlineimage:
         // Objects of this time are not directly returned.
-        ss << "pikepdf."
-           << "InlineImage";
+        ss << "pikepdf.InlineImage";
         break;
-    // LCOV_EXCL_END
+    // LCOV_EXCL_STOP
     case QPDFObject::object_type_e::ot_array:
-        ss << "pikepdf."
-           << "Array";
+        ss << "pikepdf.Array";
         break;
     case QPDFObject::object_type_e::ot_dictionary:
         if (h.hasKey("/Type")) {
-            ss << "pikepdf."
-               << "Dictionary(Type=\"" << h.getKey("/Type").getName() << "\")";
+            ss << "pikepdf.Dictionary(Type=\"" << h.getKey("/Type").getName() << "\")";
         } else {
-            ss << "pikepdf."
-               << "Dictionary";
+            ss << "pikepdf.Dictionary";
         }
         break;
     case QPDFObject::object_type_e::ot_stream:
-        ss << "pikepdf."
-           << "Stream";
+        ss << "pikepdf.Stream";
         break;
     case QPDFObject::object_type_e::ot_null:
     case QPDFObject::object_type_e::ot_boolean:
     case QPDFObject::object_type_e::ot_integer:
     case QPDFObject::object_type_e::ot_real:
-        // LCOV_EXCL_START
-        ss << "Unexpected QPDF object type: " << h.getTypeName() << ". ";
-        ss << "Objects of this type are normally converted to native Python objects.";
-        throw std::logic_error(ss.str());
-        // LCOV_EXCL_STOP
+        break; // No typename since literal is obvious and Decimal automatically adds
+               // Decimal('1.2345.')
+
     // LCOV_EXCL_START
     default:
-        ss << "Unexpected QPDF object type value: " << h.getTypeCode();
-        throw std::logic_error(ss.str());
+        throw std::logic_error(
+            std::string("Unexpected QPDF object type value: ") + h.getTypeName());
         // LCOV_EXCL_STOP
     }
 
@@ -131,11 +121,24 @@ std::string objecthandle_pythonic_typename(QPDFObjectHandle h)
 
 std::string objecthandle_repr_typename_and_value(QPDFObjectHandle h)
 {
+    auto pythonic_typename = objecthandle_pythonic_typename(h);
+    if (pythonic_typename.empty()) {
+        return objecthandle_scalar_value(h);
+    }
     return objecthandle_pythonic_typename(h) + "(" + objecthandle_scalar_value(h) + ")";
 }
 
-static std::string objecthandle_repr_inner(
-    QPDFObjectHandle h, uint depth, std::set<QPDFObjGen> *visited, bool *pure_expr)
+std::ostream &operator<<(std::ostream &os, const QPDFObjGen &og)
+{
+    os << og.getObj() << ", " << og.getGen();
+    return os;
+}
+
+static std::string objecthandle_repr_inner(QPDFObjectHandle h,
+    uint recursion_depth,
+    uint indent_depth,
+    std::set<QPDFObjGen> *visited,
+    bool *pure_expr)
 {
     StackGuard sg(" objecthandle_repr_inner");
     std::ostringstream ss;
@@ -144,14 +147,19 @@ static std::string objecthandle_repr_inner(
     if (!h.isScalar()) {
         if (visited->count(h.getObjGen()) > 0) {
             *pure_expr = false;
-            ss << "<.get_object(" << h.getObjGen().getObj() << ", "
-               << h.getObjGen().getGen() << ")>";
+            ss << "<.get_object(" << h.getObjGen() << ")>";
             return ss.str();
         }
 
         if (!(h.getObjGen() == QPDFObjGen(0, 0)))
             visited->insert(h.getObjGen());
     }
+    if (h.isPageObject() && recursion_depth >= 1 && h.isIndirect()) {
+        ss << "<Pdf.pages.from_objgen"
+           << "(" << h.getObjGen() << ")"
+           << ">";
+        return ss.str();
+    }
 
     switch (h.getTypeCode()) {
     case QPDFObject::object_type_e::ot_null:
@@ -170,22 +178,24 @@ static std::string objecthandle_repr_inner(
         // Inline image objects are automatically promoted to higher level objects
         // in parse_content_stream, so objects of this type should not be returned
         // directly.
-        ss << objecthandle_pythonic_typename(h);
-        ss << "(";
-        ss << "data=<...>";
-        ss << ")";
+        ss << objecthandle_pythonic_typename(h) << "("
+           << "data=<...>"
+           << ")";
         break;
         // LCOV_EXCL_STOP
     case QPDFObject::object_type_e::ot_array:
         ss << "[";
         {
-            bool first = true;
+            bool first_item = true;
             ss << " ";
             for (auto item : h.getArrayAsVector()) {
-                if (!first)
+                if (!first_item)
                     ss << ", ";
-                first = false;
-                ss << objecthandle_repr_inner(item, depth, visited, pure_expr);
+                first_item = false;
+                // We don't increase indent_depth when recursing into arrays,
+                // because it doesn't look right. Always increase recursion_depth.
+                ss << objecthandle_repr_inner(
+                    item, recursion_depth + 1, indent_depth, visited, pure_expr);
             }
             ss << " ";
         }
@@ -194,37 +204,42 @@ static std::string objecthandle_repr_inner(
     case QPDFObject::object_type_e::ot_dictionary:
         ss << "{"; // This will end the line
         {
-            bool first = true;
+            bool first_item = true;
             ss << "\n";
             for (auto item : h.getDictAsMap()) {
-                if (!first)
+                auto &key = item.first;
+                auto &obj = item.second;
+                if (!first_item)
                     ss << ",\n";
-                first = false;
-                ss << std::string((depth + 1) * 2, ' '); // Indent each line
-                if (item.first == "/Parent" && item.second.isPagesObject()) {
+                first_item = false;
+                ss << std::string((indent_depth + 1) * 2, ' '); // Indent each line
+                if (key == "/Parent" && obj.isPagesObject()) {
                     // Don't visit /Parent keys since that just puts every page on the
                     // repr() of a single page
-                    ss << std::quoted(item.first) << ": <reference to /Pages>";
+                    ss << std::quoted(key) << ": <reference to /Pages>";
                 } else {
-                    ss << std::quoted(item.first) << ": "
-                       << objecthandle_repr_inner(
-                              item.second, depth + 1, visited, pure_expr);
+                    ss << std::quoted(key) << ": "
+                       << objecthandle_repr_inner(obj,
+                              recursion_depth + 1,
+                              indent_depth + 1,
+                              visited,
+                              pure_expr);
                 }
             }
             ss << "\n";
         }
-        ss << std::string(depth * 2, ' '); // Restore previous indent level
-        ss << "}";
+        ss << std::string(indent_depth * 2, ' ') // Restore previous indent level
+           << "}";
         break;
     case QPDFObject::object_type_e::ot_stream:
         *pure_expr = false;
-        ss << objecthandle_pythonic_typename(h);
-        ss << "(";
-        ss << "stream_dict=";
-        ss << objecthandle_repr_inner(h.getDict(), depth + 1, visited, pure_expr);
-        ss << ", ";
-        ss << "data=<...>";
-        ss << ")";
+        ss << objecthandle_pythonic_typename(h) << "(owner=<...>, data=<...>, "
+           << objecthandle_repr_inner(h.getDict(),
+                  recursion_depth + 1,
+                  indent_depth + 1,
+                  visited,
+                  pure_expr)
+           << ")";
         break;
     // LCOV_EXCL_START
     default:
@@ -246,7 +261,7 @@ std::string objecthandle_repr(QPDFObjectHandle h)
 
     std::set<QPDFObjGen> visited;
     bool pure_expr    = true;
-    std::string inner = objecthandle_repr_inner(h, 0, &visited, &pure_expr);
+    std::string inner = objecthandle_repr_inner(h, 0, 0, &visited, &pure_expr);
     std::string output;
 
     if (h.isScalar() || h.isDictionary() || h.isArray()) {
